////////////////////////////////////////////////////////////////////////
// Test06sdcc.c
// Mochilote - www.cpcmania.com
////////////////////////////////////////////////////////////////////////

void SetMode0PixelColor(unsigned char *pByteAddress, unsigned char nColor, unsigned char nPixel)
{
	unsigned char nByte = *pByteAddress;

	if(nPixel == 0)
	{
		nByte &= 85;

		if(nColor & 1)
			nByte |= 128;

		if(nColor & 2)
			nByte |= 8;

		if(nColor & 4)
			nByte |= 32;

		if(nColor & 8)
			nByte |= 2;
	}
	else
	{
		nByte &= 170;

		if(nColor & 1)
			nByte |= 64;

		if(nColor & 2)
			nByte |= 4;

		if(nColor & 4)
			nByte |= 16;

		if(nColor & 8)
			nByte |= 1;
	}

	*pByteAddress = nByte;
}

void PutPixelMode0(unsigned char nX, unsigned char nY, unsigned char nColor)
{
	unsigned char nPixel = 0;
	unsigned int nAddress = 0xC000 + ((nY / 8) * 80) + ((nY % 8) * 2048) + (nX / 2);
	nPixel = nX % 2;

	SetMode0PixelColor((unsigned char *)nAddress, nColor, nPixel);
}

/**
 * Draws a line between two points p1(p1x,p1y) and p2(p2x,p2y).
 * This function is based on the Bresenham's line algorithm and is highly 
 * optimized to be able to draw lines very quickly. There is no floating point 
 * arithmetic nor multiplications and divisions involved. Only addition, 
 * subtraction and bit shifting are used. 
 *
 * Note that you have to define your own customized setPixel(x,y) function, 
 * which essentially lights a pixel on the screen.
 */
void swap(int n1, int n2)
{
	int nAux = n1;
	n1 = n2;
	n2 = nAux;
}
void lineBresenham(int p1x, int p1y, int p2x, int p2y)
{
    int F, x, y;
    int dy;
    int dx;
    int dy2;
    int dx2;
    int dy2_minus_dx2;
    int dy2_plus_dx2;

    if (p1x > p2x)  // Swap points if p1 is on the right of p2
    {
        swap(p1x, p2x);
        swap(p1y, p2y);
    }

    // Handle trivial cases separately for algorithm speed up.
    // Trivial case 1: m = +/-INF (Vertical line)
    if (p1x == p2x)
    {
        if (p1y > p2y)  // Swap y-coordinates if p1 is above p2
        {
            swap(p1y, p2y);
        }

        x = p1x;
        y = p1y;
        while (y <= p2y)
        {
            PutPixelMode0(x, y, 1);
            y++;
        }
        return;
    }
    // Trivial case 2: m = 0 (Horizontal line)
    else if (p1y == p2y)
    {
        x = p1x;
        y = p1y;

        while (x <= p2x)
        {
            PutPixelMode0(x, y, 1);
            x++;
        }
        return;
    }


    dy            = p2y - p1y;  // y-increment from p1 to p2
    dx            = p2x - p1x;  // x-increment from p1 to p2
    dy2           = (dy << 1);  // dy << 1 == 2*dy
    dx2           = (dx << 1);
    dy2_minus_dx2 = dy2 - dx2;  // precompute constant for speed up
    dy2_plus_dx2  = dy2 + dx2;


    if (dy >= 0)    // m >= 0
    {
        // Case 1: 0 <= m <= 1 (Original case)
        if (dy <= dx)   
        {
            F = dy2 - dx;    // initial F

            x = p1x;
            y = p1y;
            while (x <= p2x)
            {
                PutPixelMode0(x, y, 1);
                if (F <= 0)
                {
                    F += dy2;
                }
                else
                {
                    y++;
                    F += dy2_minus_dx2;
                }
                x++;
            }
        }
        // Case 2: 1 < m < INF (Mirror about y=x line
        // replace all dy by dx and dx by dy)
        else
        {
            F = dx2 - dy;    // initial F

            y = p1y;
            x = p1x;
            while (y <= p2y)
            {
                PutPixelMode0(x, y, 1);
                if (F <= 0)
                {
                    F += dx2;
                }
                else
                {
                    x++;
                    F -= dy2_minus_dx2;
                }
                y++;
            }
        }
    }
    else    // m < 0
    {
        // Case 3: -1 <= m < 0 (Mirror about x-axis, replace all dy by -dy)
        if (dx >= -dy)
        {
            F = -dy2 - dx;    // initial F

            x = p1x;
            y = p1y;
            while (x <= p2x)
            {
                PutPixelMode0(x, y, 1);
                if (F <= 0)
                {
                    F -= dy2;
                }
                else
                {
                    y--;
                    F -= dy2_plus_dx2;
                }
                x++;
            }
        }
        // Case 4: -INF < m < -1 (Mirror about x-axis and mirror 
        // about y=x line, replace all dx by -dy and dy by dx)
        else    
        {
            F = dx2 + dy;    // initial F

            y = p1y;
            x = p1x;
            while (y >= p2y)
            {
                PutPixelMode0(x, y, 1);
                if (F <= 0)
                {
                    F += dx2;
                }
                else
                {
                    x++;
                    F += dy2_plus_dx2;
                }
                y--;
            }
        }
    }
}




main()
{
	//SCR_SET_MODE 0
	__asm
		ld a, #0
		call #0xBC0E
	__endasm;

	lineBresenham(0, 0, 159, 199);
	lineBresenham(0, 199, 159, 0);
	lineBresenham(80, 0, 80, 199);
	lineBresenham(0, 100, 159, 100);

	while(1) {};
}
////////////////////////////////////////////////////////////////////////